/*
 * File:   AQM1602.c
 * Author: kitanote
 *
 * Created on December 12, 2021, 4:23 PM
 */

#include "AQM1602.h"
uint8_t     LCD_mode = 2;
char        sv_x = 0;
uint8_t     D_C_B = 0b0001100;

void writeCommand(uint8_t t_command)
{
    uint8_t     data[2];
    I2C1_MESSAGE_STATUS     pstatus;
    
    data[0] = 0x00;
    data[1] = t_command;
    I2C1_MasterWrite(data, 2, LCD_ADRS,  &pstatus);
    __delay_ms(1);
}

void writeData(uint8_t t_data)
{
    uint8_t     data[2];
    I2C1_MESSAGE_STATUS     pstatus;

    data[0] = 0x40;
    data[1] = t_data;
    I2C1_MasterWrite(data, 2, LCD_ADRS, &pstatus);
    __delay_ms(1);
}

void init_LCD() 
{
    writeCommand(0x38); // FUNCTION SET
    __delay_ms(1);
    writeCommand(0x39); // IS=1  
    __delay_ms(1);
    writeCommand(0x14); // INT OSC FREQUENCY  
    __delay_ms(1);
    writeCommand(0x73); // CONTRAST SET 0,1,2,3  
    __delay_ms(1);
    writeCommand(0x56); // CONTRAST SET 4,5
    __delay_ms(1);
    writeCommand(0x6C); // F0LLOWER CONTROL  
    __delay_ms(200);
    writeCommand(0x38); // IS=0
    __delay_ms(1);
    writeCommand(0x0C); // Display ON  
    __delay_ms(1);
    writeCommand(0x01); // Clear Display  
    __delay_ms(1);
    writeCommand(0x06); // Entry Mode  
    __delay_ms(1);
  }

void LCD_Cls() 
{
    writeCommand(CLS);
    sv_x = 0;
}

void LCD_Home()
{
    writeCommand(Home);
    sv_x = 0;
}

void LCD_Mode(int no)
{
    uint8_t mode;
    
   switch(no)
    {
        case 1:     mode = 0x30; break;
        case 3:     mode = 0x34; break; 
        default:    mode = 0x38; no = 2; 
    }
    
    LCD_mode = no;
    writeCommand(mode);    
}

void LCD_Move(uint8_t x, uint8_t y)
{
    if(LCD_mode == 2)
        if(y > 0) x += 0x40;  
    
    sv_x = x;
    x |= 0x80;
    writeCommand(x); 
}

void LCD_Print_L(uint8_t x, uint8_t y, char * data)
{
    LCD_Move(x, y);
    LCD_Print(data);
}

void LCD_Print(char * data)
{
    while(*data)
    {
        writeData(*data);
        data ++;
        sv_x ++;
        Check_Position();
    }
}

void Check_Position()
{
    int flg;
    
    flg = 0;
    if(sv_x == 0x10) { sv_x = 0; flg = 1; }
    if(sv_x == 0x50) { sv_x = 0x40; flg = 1; }
    if(sv_x < 0) sv_x = 0x0f;
    if(sv_x == 0x3f) sv_x = 0x67;
    
    if(flg) writeCommand(sv_x | 0x80); 
}

void Contrast(uint8_t data)
{
    uint8_t cont, _is;
    
    switch(LCD_mode)
    {
        case 1:     _is = 0x0; break;
        case 3:     _is = 0x4; break;
        default:    _is = 0x8;
    }
    _is |= 0x31;
    writeCommand(_is); // Set extension ON
    
    cont = data & 0xf;
    cont |= 0x70;
    writeCommand(cont); // CONTRAST SET 0,1,2,3 
    __delay_ms(1);
    cont = (data & 0x30) >> 4;
    cont |= 0x54;
    writeCommand(cont); // CONTRAST SET 4,5
    __delay_ms(1);
    
    _is &= 0xfe;
    writeCommand(_is); // Set extension OFF
}

void LCD_Put_C(char data)
{
    writeData(data);
    sv_x ++;
    Check_Position();
}

void LCD_Put_C_L(uint8_t x, uint8_t y, char data)
{
    LCD_Move(x, y);
    LCD_Put_C(data);
}

void Display(uint8_t data)
{
    uint8_t a;
    
    if(data) a = D_C_B | 0x4;
    else a = D_C_B & 0xfb;
    writeCommand(a);
}

void Cursor(uint8_t data)
{
    uint8_t a;
    
    if(data) a = D_C_B | 0x2;
    else a = D_C_B & 0xfd;
    writeCommand(a);
}

void Brink(uint8_t data)
{
    uint8_t a;
    
    if(data) a = D_C_B | 0x1;
    else a = D_C_B & 0xfe;
    writeCommand(a);
}

void Cursor_Shift(uint8_t data)
{
    if(data) sv_x ++;
    else sv_x --;
    
    Check_Position();
    writeCommand(sv_x | 0x80); 
}

void Set_Char(uint8_t no, uint8_t *data)
{
    uint8_t a;
    
    writeCommand(0x40 + (no * 8));
    for(a = 0; a < 8; a ++) writeData(data[a]);
    writeCommand(sv_x | 0x80); 
}
